const db = require('../db');
const authwrap = require('../authwrap');
const check_permission = require('../check_permission');

let newstreamid = 0;

async function stream(msg, respond, socket) {
	if (!(await check_permission(msg.auth_user.id, msg.thread)).post) {
		return respond({
			success: false,
			message: "user doesn't have permission"
		});
	}
	let stream;
	let thread = vybe.threads[msg.thread] || (vybe.threads[msg.thread] = {
		streams: []
	});
	async function send() {
		const permissions = await db.query(
			`select * from permission
			where thread = ? and type = 'everyone' and value = 'true' and permission = 'view'`,
			msg.thread);
		if (permissions.rows.length) {
			for (let id in vybe.users)
				for (let socket of vybe.users[id].sockets)
					socket.emit('stream', stream);
		} else {
			for (let member of (
				await db.query(
					'select user from member where member.thread = ?', msg.thread)
			).rows) {
				member = vybe.users[member.user];
				if (member)
					for (let socket of member.sockets)
						socket.emit('stream', stream);
			}
		}
	}
	if (msg.id) {
		stream = vybe.streams[msg.id];
		if (!stream)
			return respond({
				success: false,
				message: 'stream not found'
			});
		if (msg.auth_user.id !== stream.userid)
			return respond({
				success: false,
				message: "stream doesn't belong to user"
			});
		if (msg.stop) {
			stream.stop();
			return respond({
				success: true
			});
		}
		stream = stream.stream;
		stream.name = msg.name;
	}
	else {
		stream = {
			id: newstreamid++,
			thread: msg.thread,
			user: msg.auth_user.displayname,
			name: msg.name
		};
		thread.streams.push(stream);
		vybe.streams[stream.id] = {
			stream,
			userid: msg.auth_user.id,
			listeners: {},
			socket: socket.id,
			stop: () => {
				stream.stopped = true;
				thread.streams.splice(thread.streams.findIndex(s => s.id === stream.id), 1);
				delete vybe.streams[stream.id];
				send();
			}
		};
	}
	await send();
	respond({
		success: true,
		id: stream.id
	});
}

async function streamdata(msg, respond) {
	let stream = vybe.streams[msg.id];
	if (!stream) {
		return respond({
			success: false,
			message: 'stream not found'
		});
	}
	if (stream.userid !== msg.auth_user.id) {
		return respond({
			success: false,
			message: "stream doesn't belong to user"
		});
	}
	for (let id in stream.listeners)
		stream.listeners[id].emit('streamdata', {
			id: msg.id,
			audio: msg.audio,
			video: msg.video
		});
	respond({
		success: true
	});
}

async function play_stream(msg, respond, socket) {
	if (!(await check_permission(msg.auth_user.id, msg.thread)).view) {
		return respond({
			success: false,
			message: "user doesn't have permission"
		});
	}
	if (!vybe.streams[msg.id])
		return respond({
			success: false,
			message: 'stream not found'
		});
	if (msg.playing)
		vybe.streams[msg.id].listeners[socket.id] = socket;
	else
		delete vybe.streams[msg.id].listeners[socket.id];
	respond({
		success: true
	});
}

module.exports = {
	stream: authwrap(stream),
	streamdata: authwrap(streamdata),
	play_stream: authwrap(play_stream)
};
